    #include <stdlib.h>
    #include <stdio.h>

    #define N 3
    #define M 4

    // dati
    /*
    Gli elementi del tipo Tabella sono tabelle bidimensionali (NxM) di interi, dove N e M
    sono delle costanti.
    Gli elementi di tipo Lista sono liste di coppie di indici.
    */
    typedef int Tabella[N][M];

    typedef struct Elem {
      int riga;
      int colonna;
      struct Elem* next;
    } Elem;

    typedef Elem* Lista;

    // consegne
    /*
    1. int occorre_piu_volte(Tabella t);
    Restituisce l'intero che compare il numero maggiore di volte nella tabella
    passata come parametro attuale.
    2. Lista seleziona(Tabella t, int k);
    Restituisce la lista di tutte le coppie (i,j) tali che il valore della cella t[i][j] e'
    minore di k.
    3. void sostituisci(Tabella t, int k);
    Modifica la tabella t sostituendo tutte le occorrenze di multipli di k con k.
    */

    // 1)
    // funzione ausiliaria che conta il numero che compare più volte in un array
    int max_n (int a[], int size) {
      int i, j, n, count;
      int tot = -1;
      for (i = 0; i < size; i++) { // per ogni numero
        count = 0;
        for (j = i; j < size; j++) { // conta quante volte compare da lì in poi (prima è già stato contato)
          if (a[i] == a[j])
            count++;
        }
        if (count > tot) { // lo è sempre dopo il primo for perché tot = -1
          tot = count;
          n = a[i];
        }
      }
      return n;
    }

    int occorre_piu_volte(Tabella t) {
      int n, m;
      int i = 0;
      // per comodità porto tutto in un array ad una dimensione
      int a[N*M];
      for (n = 0; n < N; n++)
        for (m = 0; m < M; m++, i++)
          a[i] = t[n][m];
      return max_n (a, N*M);
    }


    // 2)
    // funzione ausiliaria aggiungi in lista un numero della tabella
    void add_nm_l (Lista *l, int n, int m) {
      if (l == NULL) return;
      if (*l == NULL) {
        *l = malloc (sizeof (Elem));
        (*l) -> riga = n;
        (*l) -> colonna = m;
        (*l) -> next = NULL;
        return;
      }
      return add_nm_l (&((*l) -> next), n, m);
    }

    Lista seleziona (Tabella t, int k) {
      int n, m;
      Lista ln = NULL;
      for (n = 0; n < N; n++)
        for (m = 0; m < M; m++) {
          if (t[n][m] < k)
            add_nm_l (&(ln), n, m);
        }
      return ln;
    }

    // 3
    void sostituisci (Tabella t, int k) {
      int n, m;
      for (n = 0; n < N; n++)
        for (m = 0; m < M; m++) {
          if ((t[n][m] % k) == 0)
            t[n][m] = k;
        }
      return;
    }

    // funzioni utili
    void print_t (Tabella t) {
      int n, m;
      printf ("{\n");
      for (n = 0; n < N; n++) {
        printf ("{");
        for (m = 0; m < M; m++) {
          printf ("%d, ", t[n][m]);
        }
        printf ("}, \n");
      }
      printf ("}\n");
      return;
    }

    void print_l (Lista l) {
      while (l != NULL) {
        printf ("(%d, %d) -> ", l -> riga, l -> colonna);
        l = l -> next;
      }
      printf ("NULL\n");
      return;
    }

    int main () {
      // test 1
      Tabella t = {
        {1, 0, 1, 4},
        {2, 2, 3, 2},
        {3, 5, 2, 1},
      };
      print_t (t);
      printf ("%d\n", occorre_piu_volte (t));

      // test 2
      printf ("\n");
      Tabella t2 = {
        {1, 0, 1, 4},
        {2, 2, 3, 2},
        {3, 5, 3, 1},
      };

      Lista l = seleziona (t2, 3);
      //l = seleziona (t2, 0);
      print_l (l);

      // test 3
      printf ("\n");
      Tabella t3 = {
        {1, 0, 10, 4},
        {20, 2, 3, 5},
        {3, 15, 3, 1},
      };

      print_t (t3);
      sostituisci (t3, 5);
      print_t (t3);

      return 0;
    }
