    #include <stdio.h>
    #include <stdlib.h>
    #include <string.h>

    // dati
    /*
    Il tema di oggi ci proietta nelle prossime vacanze al mare.
    I tipi che sono qui sotto elencati permettono di rappresentare lo stato delle
    prenotazioni in uno stabilimento balnerare costituito da N file ed M colonne di
    postazioni, ciascuna delle quali ha un ombrellone e due sdraio.
    Per semplicità assumiamo che lo stabilimento abbia solo due file e due colonne
    di postazioni.
    */
    #define N 2
    #define M 2
    /*
    Gli elementi del tipo Cliente sono strutture con tre campi di tipo stringa,
    contenenti rispettivamente cognome, nome ed indirizzo.
    */
    typedef struct Cliente{
      char cognome[30];
      char nome[30];
      char indirizzo[60];
    } Cliente;
    /*
    Gli elementi del tipo Stato rappresentano una "postazione" nella spiaggia,
    costituita da un ombrellone e due sdraio. Si tratta di valori di tipo struttura con
    tre campi, i cui valori sono NULL oppure l'indirizzo di un valore di tipo Cliente
    (il cliente che ha prenotato in parte o totalmente tale postazione).
    */
    typedef struct Stato {
      Cliente* sdraio1; /* NULL = non prenotato */
      Cliente* sdraio2; /* NULL = non prenotato */
      Cliente* ombrellone; /* NULL= non prenotato */
    } Stato;
    /*
    Gli elementi del tipo Mese e di tipo Data permettono di rapresentare una data
    nel periodo di apertura della spiaggia..
    */
    typedef enum Mese{Giu,Lug,Ago,Set} Mese;
    typedef struct Data{ Mese mese; int giorno;} Data;
    /*
    Gli elementi del tipo SpiaggiaGiorno sono strutture che rappresentano la
    situazione delle prenotazioni della spiaggia (che ha N righe e M colonne di
    postazioni, ovvero in totale N*M ombrelloni ed il doppio di sdraio) in un singolo
    giorno. Un valore di tipo Stagione è una lista ordinata di elementi di tipo
    SpiaggiaGiorno (intuitivamente, il campo next punta alla situazione della
    spiaggia in un giorno successivo).
    */
    typedef struct SpiaggiaGiorno{
      Stato prenotazioni[N][M];
      Data d;
      struct SpiaggiaGiorno* next;
    } SpiaggiaGiorno;

    typedef SpiaggiaGiorno* Stagione;
    /*
    Infine, gli elementi di tipo Richiesta rappresentano le possibili richieste di
    prenotazione da parte di un cliente: solo l'ombrellone, oppure ombrellone ed
    una sola sdraio, oppure ombrellone ed entrambe le sdraio di una postazione.
    */
    typedef enum Richiesta{soloOmbrellone, unaSdraioEOmbrellone,
    dueSdraioEOmbrellone} Richiesta;

    // richieste
    /*
    1. int verifica_disponibilita(Data d, Richiesta r, Stagione s);
    Questa funzione verifica se nella stagione s c'è disponibilità dei servizi
    richiesti r nella data d: in caso positivo, la funzione restituisce 1, in caso
    negativo la funzione restituisce 0.

    2. int ombrelloni_liberi(Stagione s, Data d);
    Questa funzione restituisce il numero totale di ombrelloni non ancora
    prenotati in data d nella stagione s.

    3. int annulla(Stagione s, Cliente* c);
    Questa funzione rimuove tutte le prenotazioni effettuate da parte del
    cliente c nella stagione s; la funzione restituisce 0 se il cliente non risulta
    presente, e restituisce 1 in tutti gli altri casi.
    */

    // 1) Dato che non si possono prenotare solo sdraio
    // verifico solo se l'ombrellone è libero, se lo è lo sono anche le sdraio
    int verifica_disponibilita (Data d, Richiesta r, Stagione s) {
      int i, j;
      int ris = 0;
      // scorro la stagione fino alla data richiesta, se non c'è torna ris = 0
      while (s != NULL) {
        // trovata la data scorro l'array dei vari stati
        if ((s -> d).mese == d.mese && (s -> d).giorno == d.giorno) {
          for (i = 0; i < N; i++)
            for (j = 0; j < N; j++)
              if ((s -> prenotazioni[i][j]).ombrellone == NULL) {
                return 1;
              }
        }
        s = s -> next;
      }
      return ris;
    }

    // 2)
    int ombrelloni_liberi (Stagione s, Data d) {
      int i, j;
      int tot = 0;
      while (s != NULL) {
        if ((s -> d).mese == d.mese && (s -> d).giorno == d.giorno) {
          for (i = 0; i < N; i++)
            for (j = 0; j < N; j++)
              if ((s -> prenotazioni[i][j]).ombrellone == NULL) {
                tot++;
              }
        }
        s = s -> next;
      }
      return tot;
    }

    // 3)
    int annulla (Stagione s, Cliente* c) {
      int i, j;
      int ris = 0;
      while (s != NULL) {
        for (i = 0; i < N; i++)
          for (j = 0; j < N; j++) {
            /*// stampo
            printf ("\n%d/%d: ", (s -> d).giorno, (s -> d).mese);
            if ((s -> prenotazioni[i][j]).ombrellone != NULL) { // altrimenti segmentation fault
              printf ("%s", ((s -> prenotazioni[i][j]).ombrellone) -> nome); // == (*((s -> prenotazioni[i][j]).ombrellone)).nome
            }*/
            
            // se il cliente ha prenotato l'ombrellone lo tolgo sia dall'ombrellone che dalle sdraio
            if ((s -> prenotazioni[i][j]).ombrellone == c) {
              (s -> prenotazioni[i][j]).ombrellone = NULL;
              (s -> prenotazioni[i][j]).sdraio1 = NULL;
              (s -> prenotazioni[i][j]).sdraio2 = NULL;
              ris = 1;
            }
          }
        s = s -> next;
      }
      return ris;
    }


    // funzioni utili
    // stampa cliente
    void print_c (Cliente c) {
      printf ("%s %s, %s", c.cognome, c.nome, c.indirizzo);
      return;
    }

    // crea cliente
    Cliente cliente (Cliente c, char cognome[], char nome[], char ind[]) {
      strcpy (c.cognome, cognome);
      strcpy (c.nome, nome);
      strcpy (c.indirizzo, ind);
      return c;
    }

    // copia array di stato
    void copy_st (Stato stato[N][M], Stato giorno[N][M]) {
      int i, j;
      for (i = 0; i < N; i++)
        for (j = 0; j < N; j++)
          giorno[i][j] = stato[i][j];
      return;
    }

    // aggiungi giorno in stagione, passo una data ed un array di stato
    void add_day (Stagione *s, Data d, Stato stato[N][M]) {
      if (s == NULL) return;
      if (*s == NULL) {
        *s = malloc (sizeof (SpiaggiaGiorno));
        (*s) -> d = d;
        copy_st (stato, (*s) -> prenotazioni);
        (*s) -> next = NULL;
        return;
      }
      return add_day (&((*s) -> next), d, stato);
    }

    int main () {
      // Dati di esempio
      // due clienti
      Cliente c1, c2;
      c1 = cliente (c1, "pallino", "pinco", "via verdi");
      c2 = cliente (c2, "tizio", "caio", "via rossi");

      // stato vuoto, per i posti liberi
      Stato vuoto;
      vuoto.sdraio1 = NULL;
      vuoto.sdraio2 = NULL;
      vuoto.ombrellone = NULL;

      // creo uno stato non vuoto (viste le richieste non possono esserci sdraio prenotate e ombrellone no)
      // cliente 1 con ombrellone e sdraio
      Stato st1;
      st1.sdraio1 = &c1;
      st1.sdraio2 = NULL;
      st1.ombrellone = &c1;

      // creo un altro stato
      // cliente 2 con solo ombrellone
      Stato st2;
      st2.sdraio1 = NULL;
      st2.sdraio2 = NULL;
      st2.ombrellone = &c2;

      // Due date
      Data giugno1;
      giugno1.mese = 0;
      giugno1.giorno = 1;

      Data giugno2;
      giugno2.mese = 0;
      giugno2.giorno = 2;

      // Imposto stagione (anno2011) con solo le due date
      // giugno: st1, NULL, st2, NULL
      Stagione anno2011 = NULL;
      Stato giugno1st[N][M] = {
        {st1, vuoto},
        {st2, vuoto}
      };
      Stato giugno2st[N][M] = {
        {st1, st1},
        {st2, st2}
      };
      add_day (&anno2011, giugno1, giugno1st); // il primo giugno due postazioni sono libere
      add_day (&anno2011, giugno2, giugno2st); // il due giugno sono tutte occupate dai clienti c1 e c2

      // Richiesta è di tipo enum quindi int
      // 0 = soloOmbrellone, ecc

      // test 1
      printf ("%d\n", verifica_disponibilita (giugno2, 0, anno2011));
      printf ("%d\n", verifica_disponibilita (giugno1, 0, anno2011));

      // test 2
      printf ("\n\n");
      printf ("%d\n", ombrelloni_liberi (anno2011, giugno2));
      printf ("%d\n", ombrelloni_liberi (anno2011, giugno1));

      // test 3
      printf ("\n\n");
      printf ("%d\n", annulla (anno2011, &c1)); // tolgo il cliente 1 dalla stagione
      // ora il 2 giugno ci sono posti liberi
      printf ("%d\n", verifica_disponibilita (giugno2, 0, anno2011));
      // il primo giugno c'è un ombrellone in più
      printf ("%d\n", ombrelloni_liberi (anno2011, giugno1));

      printf ("%d\n", annulla (anno2011, &c1)); // dà 0 perché ho già tolto il cliente 1

      return 0;
    }
