    #include <stdio.h>
    #include <stdlib.h>

    // dati
    /*
    Qualcuno vi ha fatto un bello scherzo, mettendo le mani nell'armadio dove tenete la vostra biancheria:
    anziche' avere calzettoni e calzini appaiati e ordinati, davanti a voi c'e' una pila di calzettoni e calzini,
    tutti messi alla rinfusa, un calzetto lungo verde seguito da una calzetto corto nero, un calzetto corto grigio
    e poi l'altro calzetto lungo verde... e quello che e' peggio e' che non per tutti si trova il corrispondende!
    Per la prova di oggi useremo i seguenti tipi:
    */

    typedef enum {Verde, Grigio, Bianco, Nero} Colore;

    typedef enum {Corto, Lungo} Lunghezza;

    typedef struct Calzetto{
      Colore colore;
      Lunghezza lunghezza;
      struct Calzetto* next;
    } Calzetto;

    typedef Calzetto* ListaCalzetti;

    /*
    Esercizio 1: Implementare la seguente funzione:
    int conta(ListaCalzetti l);
    La funzione conta restituisce il numero di paia di calzetti (della stessa lunghezza e dello stesso colore!) presenti nella lista.

    Esercizio 2: Implementare la seguente procedura:
    void spaiati(ListaCalzetti *l);
    La procedura spaiati toglie dalla lista *l tutte le coppie di calzetti correttamente abbinabili: all'uscita dela procedura in *l
    resteranno quindi solo calzetti spaiati!

    Esercizio 3: Implementare la seguente procedura:
    void statistica(ListaCalzetti l, int tabella[4][2]);
    All'uscita della procedura statistica, nell'array bidimensionale di interi tabella (dove le righe sono relative al colore
    e le colonne relative alla lunghezza) dovremo avere i dati corretti relativi ai calzetti presenti nella lista l: nella casella
    tabella[0][0] dovremo avere il numero totale di calzetti corti e verdi, nella casella tabella[1][1] il numero dei
    calzetti lunghi e grigi, nella casella tabella[3][0] il numero dei calzetti corti e neri, eccetera.
    */

    // 1) "torna le coppie non sovrapposte" di calzini, sono otto combinazioni (0, 0), (0, 1), (1, 0) ...
    int conta (ListaCalzetti l) {
      int i, j;
      int coppie = 0;
      // array delle varie combinazioni, setto a 0 altrimenti hanno un valore sporco
      int comb [4][2] = {{0, 0}, {0, 0}, {0, 0}, {0, 0}};
      // scorro la lista, incremento il val dell'array all'indice del rispettivo calzino
      while (l != NULL) {
        comb[l -> colore][l -> lunghezza]++;
        l = l -> next;
      }

      // scorro l'array sommando le coppie di calzini trovati
      for (i = 0; i < 4; i++)
        for (j = 0; j < 2; j++) 
          coppie += comb[i][j] / 2;
      return coppie;
    }

    // 2)
    // funzione ausiliaria che cerca un calzino, se lo trova lo elimina e torna 1 altrimenti torna 0
    int find_del (ListaCalzetti *l, Calzetto c) {
      if (l == NULL) return 0;
      if (*l == NULL) return 0;
      if ((*l) -> colore == c.colore && (*l) -> lunghezza == c.lunghezza) {
        ListaCalzetti aux = *l;
        *l = (*l) -> next;
        free (aux);
        return 1;
      }
      return find_del (&((*l) -> next), c);
    }

    void spaiati (ListaCalzetti *l) {
      if (l == NULL) return;
      if ((*l) == NULL) return;
      // cerca il calzino nella coda della lista
      int r = find_del (&((*l) -> next), *(*l)); // (*l) = valore lista cioè di Calzetto *, *(*l) = Calzetto
      // se lo trovi elimina anche la testa
      if (r == 1) {
        *l = (*l) -> next;
        return spaiati (l);
      }
      // altrimenti cerca spaiati nella coda della lista
      return spaiati (&((*l) -> next));
    }

    // 3)
    void statistica (ListaCalzetti l, int tabella[4][2]) {
      int i, j;
      // azzero l'array
      for (i = 0; i < 4; i++)
        for (j = 0; j < 2; j++)
          tabella[i][j] = 0;
      // scorro la lista e incremento il relativo indice dell'array
      while (l != NULL) {
        tabella[l -> colore][l -> lunghezza]++;
        l = l -> next;
      }
      return;
    }

    // funzioni utili
    // aggiungi calzino
    void add_c (ListaCalzetti *l, Colore col, Lunghezza lung) {
      if (l == NULL) return;
      if (*l == NULL) {
        *l = malloc (sizeof (Calzetto));
        (*l) -> colore = col;
        (*l) -> lunghezza = lung;
        (*l) -> next = NULL;
        return;
      }
      return add_c (&((*l) -> next), col, lung);
    }

    // stampa lista calzini
    void print_c (ListaCalzetti l) {
      while (l != NULL) {
        // stampo l'intero per il colore e la lunghezza
        printf ("(%d, %d) -> ", l -> colore, l -> lunghezza);
        l = l -> next;
      }
      printf ("NULL\n");
      return;
    }

    // stampa array 4 x 2
    void print_a (int a[4][2]) {
      int i, j;
      for (i = 0; i < 4; i++) {
        for (j = 0; j < 2; j++)
          printf ("(%d, %d) = %d  ", i, j, a[i][j]);
        printf ("\n");
      }
    }

    int main () {
      // test 1
      ListaCalzetti l = NULL;
      add_c (&l, Verde, Lungo); // lo passo con il nome
      add_c (&l, Nero, Corto);
      add_c (&l, Verde, Lungo);
      add_c (&l, Verde, Lungo);
      add_c (&l, Nero, Lungo);
      add_c (&l, Grigio, Lungo);
      add_c (&l, Nero, Corto);
      print_c (l);
      int prova = l -> colore; // i tipi enum sono associati ad un intero

      int r = conta (l);
      printf ("%d\n", r); // 3 * (0, 1) + 2 * (3, 0) = 1 coppia (0, 1) + 1 coppia (3, 0) = 2

      // test 2
      printf ("\n\n");
      print_c (l);
      spaiati (&l);
      print_c (l);
      add_c (&l, 3, 1);
      printf ("\n");
      print_c (l);
      spaiati (&l);
      print_c (l);

      // test 3
      printf ("\n\n");
      int a[4][2];
      add_c (&l, Verde, Lungo);
      add_c (&l, Nero, Lungo);
      add_c (&l, Grigio, Lungo);
      print_c (l);
      statistica (l, a);
      print_a (a);

      return 0;
    }
