    #include <stdlib.h>
    #include <stdio.h>

    // dati
    /*
    Un insieme di biglie si muove su di un terreno rettangolare non piano e delimitato (le biglie non
    possono uscire), che può essere al massimo di dimensione maxRighe * maxColonne. Le celle che
    costituiscono il terreno possono essere occupate da una sola biglia. Ad ogni cella è associata una
    quota (un numero intero che esprime l'altitudine di quel punto nel terreno di gioco).
    Ciascuna biglia è identificata da un numero progressivo che la distingue dalle altre biglie. Le biglie
    si muovono di una casella solo in orizzontale o verticale (non in diagonale) e solo verso celle di
    quota strettamente inferiore.

    STRUTTURE DATI
    Il terreno è rappresentato da una matrice di interi di avente numRighe righe e numColonne colonne.
    Ogni valore corrisponde alla quota della cella corrispondente.
    */
    #define maxRighe 100
    #define maxColonne 100

    typedef int Quota;
    typedef Quota Terreno[maxRighe][maxColonne];

    typedef struct {
    int numRighe, numColonne;
    Terreno terreno;
    } TerrenoDiGioco;

    // Ciascuna biglia è caratterizzata da una posizione (riga, colonna) nel terreno di gioco e da un
    // identificativo (id) numerico univoco che serve a distinguerla dalle altre biglie.

    typedef struct {
    int riga;
    int colonna;
    } Posizione;
    typedef struct {
    int id;
    Posizione pos;
    } Biglia;

    // Le liste utilizzate negli esercizi proposti sono definite come segue:

    typedef struct elemPosizione{
    Posizione pos;
    struct elemPosizione *succ;
    } ElemPosizione;

    typedef
    ElemPosizione *ListaPosizioni;

    typedef struct elemBiglia{
    Biglia biglia;
    struct elemBiglia *succ;
    } ElemBiglia;

    typedef ElemBiglia *ListaBiglie;

    // consegne
    /*
    1) Date due biglie, posizionate in 2 celle diverse di un terreno di gioco, verificare se c'è una cella
    nella quale entrambe possono arrivare con un solo passo (avente quota inferiore rispetto ad
    entrambe). Se SI, restituirne la posizione, in caso contrario restituire la posizione (-1,-1).
    Posizione posizioneComune(TerrenoDiGioco t, Biglia b1, Biglia b2)

    2) Dati un terreno di gioco ed una biglia, restituire la lista delle posizioni accessibili con 1 passo ed
    aventi quota strettamente inferiore rispetto alla posizione corrente della biglia
    ListaPosizioni accessibili(TerrenoDiGioco t, Biglia b)

    3) Dati un terreno di gioco ed una lista di biglie non adiacenti, togliere dalla lista le biglie che non
    possono muoversi.
    void eliminaBloccate(TerrenoDiGioco t, ListaBiglie *biglie)
    */

    // 1)
    Posizione posizioneComune (TerrenoDiGioco t, Biglia b1, Biglia b2) {
      // definisco il limite a dx e in basso
      int maxdx = t.numColonne - 1; // -1 perché indice cella array
      int maxgiu = t.numRighe - 1;
      // salvo la posizione riga e colonna delle biglie e la loro quota
      int iriga1 = b1.pos.riga;
      int icol1 = b1.pos.colonna;
      Quota q1 = t.terreno[iriga1][icol1];
      int iriga2 = b2.pos.riga;
      int icol2 = b2.pos.colonna;
      Quota q2 = t.terreno[iriga2][icol2];
      // creo gli array in cui salvo le posizioni possibili delle due biglie (max 4 = su, dx, giù, sx)
      Posizione a1[4];
      Posizione a2[4];
      // metto una posizione impossibile negli array (da non considerare quando li confronto)
      Posizione p; p.riga = -1; p.colonna = -1;
      int i, j;
      for (i = 0; i < 4; i++) {
        a1[i] = p;
        a2[i] = p;
      }
      // calcolo posizioni possibili biglia 1, se non sono fuori dal tavolo e la quota è min memorizzala nell'array
      // su
      if ((iriga1 - 1) >= 0 && t.terreno[iriga1 - 1][icol1] < q1) {
        a1[0].riga = iriga1 - 1;
        a1[0].colonna = icol1;
      }
      // dx
      if ((icol1 + 1) <= maxdx && t.terreno[iriga1][icol1 + 1] < q1) {
        a1[1].riga = iriga1;
        a1[1].colonna = icol1 + 1;
      }
      // giu
      if ((iriga1 + 1) <= maxgiu && t.terreno[iriga1 + 1][icol1] < q1) {
        a1[2].riga = iriga1 + 1;
        a1[2].colonna = icol1;
      }
      // sx
      if ((icol1 - 1) >= 0 && t.terreno[iriga1][icol1 - 1] < q1) {
        a1[3].riga = iriga1;
        a1[3].colonna = icol1 - 1;
      }

      // faccio lo stesso per biglia 2
      // su
      if ((iriga2 - 1) >= 0 && t.terreno[iriga2 - 1][icol2] < q1) {
        a2[0].riga = iriga2 - 1;
        a2[0].colonna = icol2;
      }
      // dx
      if ((icol2 + 1) <= maxdx && t.terreno[iriga2][icol2 + 1] < q1) {
        a2[1].riga = iriga2;
        a2[1].colonna = icol2 + 1;
      }
      // giu
      if ((iriga2 + 1) <= maxgiu && t.terreno[iriga2 + 1][icol2] < q1) {
        a2[2].riga = iriga2 + 1;
        a2[2].colonna = icol2;
      }
      // sx
      if ((icol2 - 1) >= 0 && t.terreno[iriga2][icol2 - 1] < q1) {
        a2[3].riga = iriga2;
        a2[3].colonna = icol2 - 1;
      }

      // ora confronto i due array scorrendo il primo e cercando la stessa posizione (diversa da -1!) nel secondo
      for (i = 0; i < 4; i++)
        for (j = 0; j < 4; j++)
          if (a1[i].riga != -1)
            if ((a1[i].riga == a2[j].riga) && (a1[i].colonna == a2[j].colonna))
              return a1[i];
      // altr. torna posizione impossibile
      return p;
    }

    // 2)
    // funzione ausiliaria aggiungi in lista posizione
    void add_pos (ListaPosizioni * l, Posizione p) {
      if (l == NULL) return;
      if (*l == NULL) {
        *l = malloc (sizeof (ElemPosizione));
        (*l) -> pos = p;
        (*l) -> succ = NULL;
        return;
      }
      return add_pos(&((*l) -> succ), p);
    }

    ListaPosizioni accessibili (TerrenoDiGioco t, Biglia b) {
      int maxdx = t.numColonne - 1;
      int maxgiu = t.numRighe - 1;
      ListaPosizioni l = NULL;
      Posizione p;
      int iriga1 = b.pos.riga;
      int icol1 = b.pos.colonna;
      Quota q1 = t.terreno[iriga1][icol1];

      // su
      if ((iriga1 - 1) >= 0 && t.terreno[iriga1 - 1][icol1] < q1) {
        p.riga = iriga1 - 1;
        p.colonna = icol1;
        add_pos (&l, p);
      }
      // dx
      if ((icol1 + 1) <= maxdx && t.terreno[iriga1][icol1 + 1] < q1) {
        p.riga = iriga1;
        p.colonna = icol1 + 1;
        add_pos (&l, p);
      }
      // giu
      if ((iriga1 + 1) <= maxgiu && t.terreno[iriga1 + 1][icol1] < q1) {
        p.riga = iriga1 + 1;
        p.colonna = icol1;
        add_pos (&l, p);
      }
      // sx
      if ((icol1 - 1) >= 0 && t.terreno[iriga1][icol1 - 1] < q1) {
        p.riga = iriga1;
        p.colonna = icol1 - 1;
        add_pos (&l, p);
      }

      return l;
    }

    // 3)
    // funzione ausiliaria che data una biglia ed un terreno dà 1 solo se può muoversi, altr dà 0
    int move (TerrenoDiGioco t, Biglia b) {
      int maxdx = t.numColonne - 1;
      int maxgiu = t.numRighe - 1;
      int iriga1 = b.pos.riga;
      int icol1 = b.pos.colonna;
      Quota q1 = t.terreno[iriga1][icol1];

      // su
      if ((iriga1 - 1) >= 0 && t.terreno[iriga1 - 1][icol1] < q1)
        return 1;
      // dx
      if ((icol1 + 1) <= maxdx && t.terreno[iriga1][icol1 + 1] < q1)
        return 1;
      // giu
      if ((iriga1 + 1) <= maxgiu && t.terreno[iriga1 + 1][icol1] < q1)
        return 1;
      // sx
      if ((icol1 - 1) >= 0 && t.terreno[iriga1][icol1 - 1] < q1)
        return 1;

      return 0; // se non può muoversi in nessuna direzione
    }

    void eliminaBloccate (TerrenoDiGioco t, ListaBiglie *biglie) {
      if (biglie == NULL) return;
      if (*biglie == NULL) return;
      if (move (t, (*biglie) -> biglia) == 0) {
        ListaBiglie aux = *biglie;
        *biglie = (*biglie) -> succ;
        free (aux);
        return eliminaBloccate (t, biglie);
      }
      else return eliminaBloccate (t, &((*biglie) -> succ));
    }


    // funzioni utili
    // stampa posizione
    void print_p (Posizione p) {
      printf ("(%d, %d)", p.riga, p.colonna);
      return;
    }
    // stampa lista posizioni
    void print_lp (ListaPosizioni l) {
      while (l != NULL) {
        print_p (l -> pos);
        printf (" -> ");
        l = l -> succ;
      }
      printf ("NULL\n");
    }
    // stampa lista biglie
    void print_lb (ListaBiglie l) {
      while (l != NULL) {
        //print_p (l -> pos);
        printf ("%d -> ", (l -> biglia).id);
        l = l -> succ;
      }
      printf ("NULL\n");
    }
    // aggiungi biglia in lista biglie
    void add_b (ListaBiglie * l, Biglia b) {
      if (l == NULL) return;
      if (*l == NULL) {
        *l = malloc (sizeof (ElemBiglia));
        (*l) -> biglia = b;
        (*l) -> succ = NULL;
        return;
      }
      return add_b (&((*l) -> succ), b);
    }

    // test
    int main () {
      int i, j;
      int k = 0;
      // Creo terreno riempiendone solo un pezzo 3 * 3
      Terreno t;
      for (i = 0; i < 3; i++)
        for (j = 0; j < 3; j++, k++)
          t[i][j] = k; // 0, 1 ... 8
      // creo terreno di gioco 3 * 3
      TerrenoDiGioco tg;
      tg.numRighe = 3;
      tg.numColonne = 3;
      // copio i valori del terreno t nel terreno del terreno di gioco
      for (i = 0; i < 3; i++)
        for (j = 0; j < 3; j++, k++)
          tg.terreno[i][j] = t[i][j];

      // metto la prima biglia nella casella (1, 1) cioè al centro
      Biglia b1;
      b1.id = 1;
      Posizione p1; p1.riga = 1; p1.colonna = 1;
      b1.pos = p1;
      // metto la seconda biglia a dx e in basso rispetto alla prima
      Biglia b2;
      b2.id = 2;
      Posizione p2; p2.riga = 2; p2.colonna = 0;
      b2.pos = p2;

      print_p (posizioneComune (tg, b1, b2));

      // test 2
      printf ("\n\n");
      ListaPosizioni lp = accessibili (tg, b1);
      print_lp (lp);

      // test 3
      printf ("\n\n");
      ListaBiglie lb = NULL;
      add_b (&lb, b1);
      // blocco la biglia b2 nella cella in alto a sx
      p2.riga = 0; p2.colonna = 0;
      b2.pos = p2;
      add_b (&lb, b2);
      print_lb (lb);
      eliminaBloccate (tg, &lb);
      print_lb (lb);


      return 0;
    }
