    #include <stdlib.h>
    #include <stdio.h>

    // dati
    /*
    I tempi di partenza e di arrivo dei ciclisti che hanno partecipato a una gara a cronometro sono memorizzati
    in una tabella: l'elemento i-esimo di questa tabella è una coppia di valori, corrispondenti all'orario di
    partenza e all'orario di arrivo del corridore i-esimo.
    */

    #define N 5

    typedef struct {
      int ora;
      int minuto;
      int secondo;
    } Orario;

    typedef struct {
      Orario partenza;
      Orario arrivo;
    } Tempi;

    typedef Tempi Risultati[N];

    // consegne
    /*
    Esercizio 1: void ordina(Risultati ris, int dim);
    La procedura deve ordinare la tabella dei risultati ris (di dimensione dim), secondo l'ordine seguente: la coppia
    [partenza_i,arrivo_i] è minore o uguale di [partenza_j,arrivo_j] se partenza_i precede strettamente partenza_j oppure se
    partenza_i è uguale a partenza_j e arrivo_i precede (o è uguale ad) arrivo_j.

    Esercizio 2: int intersezione_zero(Risultati ris, int dim);
    La funzione intersezione_zero deve restituire 1 se nel percorso di gara c'era solo un ciclista per volta, ovvero se
    non c'è sovrapposizione alcuna tra gli intervalli di tempo presenti nella tabella ris. La funzione deve restituire 0 in caso
    contrario. Non si può verificare la condizione considerando tutte le coppie.

    Esercizio 3: int maglianera(Risultati ris, int dim);
    La funzione maglianera deve restituire 1 se dalla tabella ris risulta esserci un ciclista che è partito prima di tutti gli
    altri ed è arrivato dopo che tutti gli altri sono già arrivati. In caso contrario la procedura deve restituire 0.
    */

    // funzione ausiliaria per tutti gli es
    // orario minore, dà 1 se o1 < o2, 0 se o1 = o2, -1 se o1 > o2
    int o_min (Orario o1, Orario o2) {
      if (o1.ora < o2.ora) return 1;
      if (o1.ora > o2.ora) return -1;
      // ore uguali
      if (o1.minuto < o2.minuto) return 1;
      if (o1.minuto > o2.minuto) return -1;
      // anche minuti uguali
      if (o1.secondo < o2.secondo) return 1;
      if (o1.secondo > o2.secondo) return -1;
      // anche secondi uguali
      return 0;
    }

    // 1)
    // funzione ausiliaria per ordinamento
    void swap (Tempi * a, Tempi * b) {
      Tempi temp = *a;
      *a = *b;
      *b = temp;
      return;
    }

    // funziona come ordinamento banale
    void ordina (Risultati ris, int dim) {
      int i, j;
      for (i = 0; i < dim - 1; i++)
        for (j = 0; j < dim - 1; j++)
          // swap se partenza è maggiore oppure se è uguale ma l'arrivo è maggiore
          if (
          (o_min(ris[j].partenza, ris[j + 1].partenza) == -1) ||
          ((o_min(ris[j].partenza, ris[j + 1].partenza) == 0) && (o_min(ris[j].arrivo, ris[j + 1].arrivo) == -1))
          )
            swap (&(ris[j]), &(ris[j + 1]));
      return;
    }

    // 2)
    int intersezione_zero (Risultati ris, int dim) {
      int i;
      // ordino l'array con la funzione ordina dell'es 1
      ordina (ris, dim);
      // scorro l'array confrontando l'arrivo di un ciclista con la partenza del successivo
      // li confronto a coppie quindi dim - 1 volte
      for (i = 0; i < dim - 1; i++)
        if (o_min (ris[i].arrivo, ris[i + 1].partenza) == -1)
          return 0; // alla prima sovrapposizione esci, altrimenti alla fine torna 1
      return 1;
    }

    // 3)
    int maglianera (Risultati ris, int dim) {
      int i;
      ordina (ris, dim);
      // considero il tempo di arrivo del primo ciclista (quello partito prima)
      // lo confronto con il tempo di arrivo dei ciclisti successivi, da 1 in poi
      for (i = 1; i < dim; i++)
        // se l'arrivo è minore (1) o uguale (0) a quello di un altro esci
        if (o_min (ris[0].arrivo, ris[i].arrivo) >= 0) 
          return 0;
      return 1;
    }

    // funzioni utili
    // stampa orario
    void print_o (Orario o) {
      printf ("%d : %d : %d", o.ora, o.minuto, o.secondo);
      return;
    }
    // stampa tempi
    void print_r (Risultati r) {
      int i;
      for (i = 0; i < N; i++) {
        printf ("(");
        print_o (r[i].partenza);
        printf (", ");
        print_o (r[i].arrivo);
        printf (")\n");
      }
      return;
    }

    int main () {
      // test
      // imposto a mano 5 corridori con tempi utili per la verifica
      Orario p1, a1, p2, a2, p3, a3, p4, a4, p5, a5;
      p1.ora = 14; p1.minuto = 1; p1.secondo = 0;
      a1.ora = 14; a1.minuto = 58; a1.secondo = 5;
      p2.ora = 14; p2.minuto = 2; p2.secondo = 0;
      a2.ora = 14; a2.minuto = 57; a2.secondo = 0;
      p3.ora = 14; p3.minuto = 0; p3.secondo = 0;
      a3.ora = 14; a3.minuto = 59; a3.secondo = 57;
      p4.ora = 13; p4.minuto = 59; p4.secondo = 0; // maglia nera, parte per primo arriva per ultimo
      a4.ora = 15; a4.minuto = 00; a4.secondo = 00;
      p5.ora = 14; p5.minuto = 0; p5.secondo = 0;
      a5.ora = 14; a5.minuto = 59; a5.secondo = 59;

      Tempi t1, t2, t3, t4, t5;
      t1.partenza = p1; t1.arrivo = a1;
      t2.partenza = p2; t2.arrivo = a2;
      t3.partenza = p3; t3.arrivo = a3;
      t4.partenza = p4; t4.arrivo = a4;
      t5.partenza = p5; t5.arrivo = a5;

      Risultati r = {t1, t2, t3, t4, t5};

      // test 1
      print_r (r);
      ordina (r, N);
      printf("\n");
      print_r (r);

      // test 3
      printf ("\n\n");
      printf ("%d\n", maglianera (r, N));

      // test 2
      // partenze senza sovrapposizioni
      p1.ora = 14; p1.minuto = 0; p1.secondo = 0;
      a1.ora = 14; a1.minuto = 5; a1.secondo = 0;
      p2.ora = 14; p2.minuto = 5; p2.secondo = 0;
      a2.ora = 14; a2.minuto = 10; a2.secondo = 0;
      p3.ora = 14; p3.minuto = 10; p3.secondo = 0;
      a3.ora = 14; a3.minuto = 15; a3.secondo = 0;
      p4.ora = 14; p4.minuto = 17; p4.secondo = 0;
      a4.ora = 14; a4.minuto = 20; a4.secondo = 0;
      p5.ora = 14; p5.minuto = 20; p5.secondo = 0;
      a5.ora = 14; a5.minuto = 25; a5.secondo = 0;

      t1.partenza = p1; t1.arrivo = a1;
      t2.partenza = p2; t2.arrivo = a2;
      t3.partenza = p3; t3.arrivo = a3;
      t4.partenza = p4; t4.arrivo = a4;
      t5.partenza = p5; t5.arrivo = a5;

      r[0] = t1;
      r[1] = t2;
      r[2] = t3;
      r[3] = t4;
      r[4] = t5;

      printf ("\n\n");
      print_r (r);
      printf ("%d\n", intersezione_zero(r, N));

      return 0;
    }
